@@ -398,7 +398,7 @@ class MultiDict(TypeConversionDict[K, V]):
         return self.__class__(deepcopy(self.to_dict(flat=False), memo))
 
     @t.overload
-    def to_dict(self) -> dict[K, V]: ...
+    def to_dict(self, flat: t.Literal[True] = ...) -> dict[K, V]: ...
     @t.overload
     def to_dict(self, flat: t.Literal[False]) -> dict[K, list[V]]: ...
     def to_dict(self, flat: bool = True) -> dict[K, V] | dict[K, list[V]]:
