@@ -13,13 +13,16 @@ import uuid
 from contextlib import ExitStack
 from io import BytesIO
 from itertools import chain
+from multiprocessing import Value
 from os.path import basename
 from os.path import join
 from zlib import adler32
 
 from .._internal import _log
 from ..exceptions import NotFound
+from ..exceptions import SecurityError
 from ..http import parse_cookie
+from ..sansio.utils import host_is_trusted
 from ..security import gen_salt
 from ..utils import send_file
 from ..wrappers.request import Request
@@ -82,7 +85,8 @@ def get_machine_id() -> str | bytes | None:
         try:
             # subprocess may not be available, e.g. Google App Engine
             # https://github.com/pallets/werkzeug/issues/925
-            from subprocess import Popen, PIPE
+            from subprocess import PIPE
+            from subprocess import Popen
 
             dump = Popen(
                 ["ioreg", "-c", "IOPlatformExpertDevice", "-d", "2"], stdout=PIPE
@@ -110,7 +114,7 @@ def get_machine_id() -> str | bytes | None:
                     guid, guid_type = winreg.QueryValueEx(rk, "MachineGuid")
 
                     if guid_type == winreg.REG_SZ:
-                        return guid.encode("utf-8")
+                        return guid.encode()
 
                     return guid
             except OSError:
@@ -169,7 +173,8 @@ def get_pin_and_cookie_name(
         # App Engine. It may also raise a KeyError if the UID does not
         # have a username, such as in Docker.
         username = getpass.getuser()
-    except (ImportError, KeyError):
+    # Python >= 3.13 only raises OSError
+    except (ImportError, KeyError, OSError):
         username = None
 
     mod = sys.modules.get(modname)
@@ -193,7 +198,7 @@ def get_pin_and_cookie_name(
         if not bit:
             continue
         if isinstance(bit, str):
-            bit = bit.encode("utf-8")
+            bit = bit.encode()
         h.update(bit)
     h.update(b"cookiesalt")
 
@@ -258,7 +263,7 @@ class DebuggedApplication:
         Added the ``werkzeug.debug.preserve_context`` environ key.
     """
 
-    _pin: str
+    _pin: str | None
     _pin_cookie: str
 
     def __init__(
@@ -283,7 +288,7 @@ class DebuggedApplication:
         self.console_init_func = console_init_func
         self.show_hidden_frames = show_hidden_frames
         self.secret = gen_salt(20)
-        self._failed_pin_auth = 0
+        self._failed_pin_auth = Value("B")
 
         self.pin_logging = pin_logging
         if pin_security:
@@ -297,6 +302,14 @@ class DebuggedApplication:
         else:
             self.pin = None
 
+        self.trusted_hosts: list[str] = [".localhost", "127.0.0.1"]
+        """List of domains to allow requests to the debugger from. A leading dot
+        allows all subdomains. This only allows ``".localhost"`` domains by
+        default.
+
+        .. versionadded:: 3.0.3
+        """
+
     @property
     def pin(self) -> str | None:
         if not hasattr(self, "_pin"):
@@ -305,8 +318,12 @@ class DebuggedApplication:
         return self._pin
 
     @pin.setter
-    def pin(self, value: str) -> None:
-        self._pin = value
+    def pin(self, value: str | None) -> None:
+        if value is None:
+            # Set _pin to None explicitly to prevent regeneration by the getter
+            self._pin = None
+        else:
+            self._pin = value
 
     @property
     def pin_cookie_name(self) -> str:
@@ -343,7 +360,7 @@ class DebuggedApplication:
 
             is_trusted = bool(self.check_pin_trust(environ))
             html = tb.render_debugger_html(
-                evalex=self.evalex,
+                evalex=self.evalex and self.check_host_trust(environ),
                 secret=self.secret,
                 evalex_trusted=is_trusted,
             )
@@ -364,13 +381,16 @@ class DebuggedApplication:
 
             environ["wsgi.errors"].write("".join(tb.render_traceback_text()))
 
-    def execute_command(  # type: ignore[return]
+    def execute_command(
         self,
         request: Request,
         command: str,
         frame: DebugFrameSummary | _ConsoleFrame,
     ) -> Response:
         """Execute a command in a console."""
+        if not self.check_host_trust(request.environ):
+            return SecurityError()  # type: ignore[return-value]
+
         contexts = self.frame_contexts.get(id(frame), [])
 
         with ExitStack() as exit_stack:
@@ -381,6 +401,9 @@ class DebuggedApplication:
 
     def display_console(self, request: Request) -> Response:
         """Display a standalone shell."""
+        if not self.check_host_trust(request.environ):
+            return SecurityError()  # type: ignore[return-value]
+
         if 0 not in self.frames:
             if self.console_init_func is None:
                 ns = {}
@@ -419,6 +442,11 @@ class DebuggedApplication:
         """
         if self.pin is None:
             return True
+
+        # If we failed too many times, then we're locked out.
+        if self._failed_pin_auth.value >= 10:
+            return False
+
         val = parse_cookie(environ).get(self.pin_cookie_name)
         if not val or "|" not in val:
             return False
@@ -433,12 +461,21 @@ class DebuggedApplication:
             return None
         return (time.time() - PIN_TIME) < ts
 
+    def check_host_trust(self, environ: WSGIEnvironment) -> bool:
+        return host_is_trusted(environ.get("HTTP_HOST"), self.trusted_hosts)
+
     def _fail_pin_auth(self) -> None:
-        time.sleep(5.0 if self._failed_pin_auth > 5 else 0.5)
-        self._failed_pin_auth += 1
+        with self._failed_pin_auth.get_lock():
+            count = self._failed_pin_auth.value
+            self._failed_pin_auth.value = count + 1
+
+        time.sleep(5.0 if count > 5 else 0.5)
 
     def pin_auth(self, request: Request) -> Response:
         """Authenticates with the pin."""
+        if not self.check_host_trust(request.environ):
+            return SecurityError()  # type: ignore[return-value]
+
         exhausted = False
         auth = False
         trust = self.check_pin_trust(request.environ)
@@ -459,7 +496,7 @@ class DebuggedApplication:
             auth = True
 
         # If we failed too many times, then we're locked out.
-        elif self._failed_pin_auth > 10:
+        elif self._failed_pin_auth.value >= 10:
             exhausted = True
 
         # Otherwise go through pin based authentication
@@ -467,7 +504,7 @@ class DebuggedApplication:
             entered_pin = request.args["pin"]
 
             if entered_pin.strip().replace("-", "") == pin.replace("-", ""):
-                self._failed_pin_auth = 0
+                self._failed_pin_auth.value = 0
                 auth = True
             else:
                 self._fail_pin_auth()
@@ -488,8 +525,11 @@ class DebuggedApplication:
             rv.delete_cookie(self.pin_cookie_name)
         return rv
 
-    def log_pin_request(self) -> Response:
+    def log_pin_request(self, request: Request) -> Response:
         """Log the pin if needed."""
+        if not self.check_host_trust(request.environ):
+            return SecurityError()  # type: ignore[return-value]
+
         if self.pin_logging and self.pin is not None:
             _log(
                 "info", " * To enable the debugger you need to enter the security pin:"
@@ -516,7 +556,7 @@ class DebuggedApplication:
             elif cmd == "pinauth" and secret == self.secret:
                 response = self.pin_auth(request)  # type: ignore
             elif cmd == "printpin" and secret == self.secret:
-                response = self.log_pin_request()  # type: ignore
+                response = self.log_pin_request(request)  # type: ignore
             elif (
                 self.evalex
                 and cmd is not None
